/*
 NYTimes API Classes
 February, 2009
 blprnt@blprnt.com
 NOTE: Make sure you enter your NYTimes Article Search API on the TimesArticleSearch tab.
 */
import processing.opengl.*;
import org.json.*;
import java.util.ArrayList;

PFont font1 = createFont("Helvetica", 15); 
PFont font2 = createFont("Helvetica", 30);
int[][] transactionsPerMonth = new int[7][12];
int[][] articlesPerMonth = new int[7][12];
int[][] frontPagePerMonth = new int[7][12];
String[][] articleTitles = new String[7][12];


boolean showGrid = false;
boolean leftMouseClicked = false;
boolean rightMouseClicked = false;
boolean addColor = false;
int displayMode = 1;
int padding = 10;
float horizontalGridDivisions = 7;
float verticalGridDivisions = 12;
float horizontalSquareSize;
float verticalSquareSize;

void setup() {
  size(900, 640); //size of the sketch window
  horizontalSquareSize = width/horizontalGridDivisions;
  verticalSquareSize = height/verticalGridDivisions;
  colorMode(HSB, 360, 100, 100, 100);  
  smooth();
  
  //import the data from our text file 
  String savedData[] = loadStrings("savedData.txt");
  println("QUERY: " + savedData[0]);
  for (int i=1; i < savedData.length; i++) {
    String temp[] = split(savedData[i], ",");
    transactionsPerMonth[Integer.parseInt(temp[0])-2006][Integer.parseInt(temp[1])-1] = Integer.parseInt(temp[2]);
  } 
  
  //ny times api stuff
  for(int i = 0;i<6;i++){
    for(int j = 0;j<12;j++){
      try {
       Thread.sleep(100);
      } 
      catch (InterruptedException e) { }
      TimesArticleSearch s = new TimesArticleSearch();
      s.addQueries("obama");
      s.addFacets("page_facet");
      s.addFacetQuery("publication_year", "" + (2006+i));
      //we have to turn the month into a string like "07" which is kind of annoying. 
      String m = (j + 1 < 10) ? ("0" + str(int(j) + 1)):str(j + 1);
      s.addFacetQuery("publication_month", m);
      TimesArticleSearchResult r = s.doSearch();
      TimesFacetObject[] pageFacets = r.getFacetList("page_facet");
      println ("There were " + r.total + " articles.");
      println ("There were " + pageFacets[0].count + " articles on the front page");
      println ("The first article was titled: " + r.results[0].title);
      articlesPerMonth[i][j] = r.total;
      articleTitles[i][j] = r.results[0].title;
      frontPagePerMonth[i][j] = pageFacets[0].count;
    }
  }
}

void draw(){
  background(207,38,75);
  if(showGrid){drawGrid();}
  
  float circleDivision = (PI*2.0)/72.0;
  float horizontalDivision = (width-(padding*2))/72.0;//(horizontalSquareSize*7.0+padding)/60.0;
  
  for(int i = 0;i<6;i++){
    fill(0,100,50,100);
    strokeWeight(1.0);
    for(int j = 0;j<12;j++){
     
     //id displayMode is 1, draw bar diagram
     if(displayMode == 1){
       //SPL transactions line
       strokeWeight(1.5);
       stroke(0,100,100, 40+(72.0/width)*(width-mouseX));
       fill(0,100,100,40+(72.0/width)*mouseX);
       line(padding+horizontalDivision*((12*i)+j), height-(verticalSquareSize-padding), padding+horizontalDivision*((12*i)+j), height-(transactionsPerMonth[i][j]*0.4+verticalSquareSize-padding));
       
       //NYtimes line
       strokeWeight(1.5);
       stroke(0,0,90,20+(72.0/width)*mouseX);
       fill(0,0,90,40+(72.0/width)*mouseX);
       line(padding+horizontalDivision*((12*i)+j)+2, height-(verticalSquareSize-padding), padding+horizontalDivision*((12*i)+j)+2, height-(articlesPerMonth[i][j]*0.35+verticalSquareSize-padding));
       //ellipse(padding+horizontalDivision*((12*i)+j)+2, height-(articlesPerMonth[i][j]*0.35+verticalSquareSize-padding), 0.5*sqrt(frontPagePerMonth[i][j]), 0.5*sqrt(frontPagePerMonth[i][j]));
       
       //draw the year labels every 12 months
       if(j==0){
         textFont(font1);
         fill(0,0,15);
         textAlign(LEFT,BOTTOM);
         text(2006+i,padding+horizontalDivision*((12*i)+j), height-(padding+horizontalDivision));
       }
     
      }
   
     //if displayMode is 2 draw radial diagram 
     if(displayMode == 2){
       pushMatrix();
       translate(width/2,height/2);
       rotate(circleDivision*(i*12+j)+PI);
       stroke(0,100,100, 40+(72.0/width)*(width-mouseX));
       line(0,0,0,transactionsPerMonth[i][j]*0.4);
       rotate(0.02);
       stroke(0,0,100,40+(72.0/width)*mouseX);
       line(0,0,0,articlesPerMonth[i][j]*0.35);
       if(j==0){
         translate(0,100);
         rotate(-1*PI);
         textFont(font1);
         fill(0,100,60);
         textAlign(CENTER,CENTER);
         text(2006+i,0,0);
       } 
       popMatrix();
     }   
      
    }
  }
  
  //draw your title and labels
  textFont(font2);
  fill(0,0,15);
  textAlign(RIGHT,TOP);
  text("Titles Including \"Obama\"", width-padding, padding+verticalSquareSize);
  
  textFont(font2);
  fill(0,100,90, 40+(72.0/width)*(width-mouseX));//70+(30.0/width)*(width-mouseX)
  textAlign(LEFT,TOP);
  text("Seattle Public Library", padding, verticalSquareSize+horizontalDivision);
  
  fill(0,0,100,40+(72.0/width)*mouseX);//
  textAlign(LEFT,TOP);
  text("New York Times", (padding+2), (horizontalDivision+padding));
  
}



//Grid stuff
void drawGrid(){
  //draw horizontal grid
  stroke(0,0,35);
  strokeWeight(0.5);
  for(int i = 0;i<=horizontalGridDivisions;i++){
    line(i*horizontalSquareSize-padding,0,i*horizontalSquareSize-padding, height);
    line(i*horizontalSquareSize+padding,0,i*horizontalSquareSize+padding, height);
  }
  //draw vertical grid
  stroke(0,0,35);
  strokeWeight(0.5);
  for(int i = 0;i<=verticalGridDivisions;i++){
    line(0,i*verticalSquareSize-padding,width, i*verticalSquareSize-padding);
    line(0,i*verticalSquareSize+padding,width, i*verticalSquareSize+padding);
  }
}

//controls
void keyPressed(){
 //press spacebar to toggle the grid on and off  
 if(key == ' '){
  showGrid = !showGrid;
 }
 if(key == '1'){
   displayMode = 1;
 } 
 if(key == '2'){
   displayMode = 2;
 }
}

void star(int x1, int y1, int numSpikes, int starSize){
    //translate( x1, y1 );
    PVector[] points = new PVector[ numSpikes * 2 ];
    float innerRadius = starSize;
    float outerRadius = starSize*3;
    float angle = TWO_PI / points.length;
    for ( int i = 0; i < points.length; i++ ) {
        float x, y;
        if ( i % 2 == 0 ) {
            x = x1+cos( angle * i ) * outerRadius;
            y = y1+sin( angle * i ) * outerRadius;
        } else {
            x = x1+cos( angle * i ) * innerRadius;
            y = y1+sin( angle * i ) * innerRadius;
        }
        points[i] = new PVector( x, y );
    }
    
    beginShape();
    for (int i = 0; i < points.length; i++) {
        vertex( points[i].x, points[i].y );
    }
    endShape(CLOSE);
}

